/******************************************************************************
*******************************************************************************

	Space Impakto DS
	Relminator
	http://rel.betterwebber.com


	Homing missile


*******************************************************************************
******************************************************************************/

#include "HomingMissile.h"

void HomingMissile::Init( s32 sx, s32 sy, int frame, int irotation )
{
	Active = true;
	NewTarget = true;
	InitialRotation = irotation;
	Counter = 0;
	IsDone = false;
	Angle = BRAD_PI;
	Delta = 512;
	x = sx;
	y = sy;
	
	Speed = HOMING_SPEED;
	Dx = mulf32(cosLerp(Angle), Speed);
	Dy = mulf32(sinLerp(Angle), Speed);

	Frame = frame;
	NumFrames = 3;
	BaseFrame = 0;
	Width = 24;
	Height = 20;
	
	for( int i = 0; i < MAX_SEGMENTS; i++ )
	{
		Segments[i].x = x;
		Segments[i].y = y;
	}
	
}

void HomingMissile::Update()
{
	
	Dx = mulf32(cosLerp(Angle), Speed);
	Dy = mulf32(sinLerp(Angle), Speed);

	x += Dx;
	y += Dy;
	
	if( !IsDone )
	{
		// distance
		if ( (abs(x - Target.x) + abs(y - Target.y)) < (16 << 12) )
		{
				NewTarget = true;
				Delta = 512;  // reset initial rotation speed
		}
		
		// Project normal
		s32 dot = (mulf32(Dy , (Target.x - x)) - mulf32(Dx, (Target.y - y)));
		
		// target when counter is > 32
		if( Counter > 32 )
		{
			if( dot > 0)
				Angle = (Angle - Delta);
			else
				Angle = (Angle + Delta);
			
			if( Speed < ( HOMING_SPEED * 2 ) )
			{
				Delta += 4;  // inc delta so that we home aggressively
				Speed += 128; // inc speed to make effect cooler
			}
		
		}
		else  // rotate from back to front
		{
			Angle += Delta * InitialRotation;
		}
		
		
	}   // IsDone

	if( x  > ((SCREEN_WIDTH + 128) << 12) || x  < (( -128) << 12) )
	{
		Kill();
	}
	
	if( y  > ((SCREEN_HEIGHT + 128) << 12) || y  < (( -128) << 12) )
	{
		Kill();
	}
	
	// Animate
	if( (Counter & 3) == 0 )
	{
		Frame++;
		if( Frame > 2) Frame = 0;
	}
	
	
	// set up our segments
	Segments[MAX_SEGMENTS - 1].x = x;
	Segments[MAX_SEGMENTS - 1].y = y;

	for( int i = 0; i < (MAX_SEGMENTS - 1);  i++ )
	{
		Segments[i].x = Segments[i+1].x;
		Segments[i].y = Segments[i+1].y;
	}

	Counter++;
}

void HomingMissile::DrawLinks( int Color, glImage* const Sprites )
{
	
	Links.clear();
	
	const int STEP_SIZE = 4;

	// Calculate links
	for( int i = 0; i < (MAX_SEGMENTS-1); i += STEP_SIZE )
	{
		
		Vector2df32 Normal = GetNormalf32( Segments[i].x, Segments[i].y,
										   Segments[i + 1].x, Segments[i + 1].y,
										   false );
		Vector2df32Normalize(Normal);
		
		Link Segment;
		Segment.X = Segments[i].x;
		Segment.Y = Segments[i].y;
		Segment.Normal = Normal;
		
		Links.push_back(Segment);
		
	}
	
	// Give the darn 3D layer a different ID so that bombs would be rendered fine
	glPolyFmt( POLY_ALPHA(15) | POLY_CULL_NONE | POLY_ID( GlobalManager::Instance()->GetFreePolyID() ) );
	
	// Draw links
	for( unsigned int i = 0; i < (Links.size()-1); i++ )
	{
		s32 Lx1 = Links[i].Normal.x * 3;
		s32 Ly1 = Links[i].Normal.y * 3;
		s32 Lx2 = Links[i+1].Normal.x * 3;
		s32 Ly2 = Links[i+1].Normal.y * 3;
		int x1 = (Links[i].X + Lx1) >> 12;
		int y1 = (Links[i].Y + Ly1) >> 12;
		int x2 = (Links[i].X - Lx1) >> 12;
		int y2 = (Links[i].Y - Ly1) >> 12;
		int x3 = (Links[i+1].X - Lx2) >> 12;
		int y3 = (Links[i+1].Y - Ly2) >> 12;
		int x4 = (Links[i+1].X + Lx2) >> 12;
		int y4 = (Links[i+1].Y + Ly2) >> 12;
		
		//glTriangleFilled( x1, y1, x2, y2, x3, y3, Color );
		//glTriangleFilled( x3, y3, x4, y4, x1, y1, Color );
		//glQuadFilled( x1, y1, x2, y2, x3, y3, x4, y4, Color );
		
		
		glColor( Color );
		if( (i == 0) )
		{
			glSpriteOnQuadClipped( x1, y1, x2, y2, 
								   x3, y3, x4, y4,
								   0, 0,
								   15, 0,
								   GL_FLIP_NONE, &Sprites[0]);
		}
		else if( (i == (Links.size()-2)) )
		{
			glSpriteOnQuadClipped( x1, y1, x2, y2, 
								   x3, y3, x4, y4,
								   15, 0,
								   0, 0,
								   GL_FLIP_NONE, &Sprites[0]);
		}
		else
		{
			glSpriteOnQuadClipped( x1, y1, x2, y2, 
								   x3, y3, x4, y4,
								   15, 0,
								   15, 0,
								   GL_FLIP_NONE, &Sprites[0]);
		}


	}
	
	glColor( RGB15(31,31,31) );
			
	glPolyFmt( POLY_ALPHA(31) | POLY_CULL_NONE | POLY_ID( GlobalManager::Instance()->GetFreePolyID() ) );
	
	
	
}
