/******************************************************************************
*******************************************************************************

	Bubble Fight EX
	relminator
	http://rel.betterwebber.com


	Plato class


*******************************************************************************
******************************************************************************/

#include "Plato.h"

#define MIN_X ( -20 << 12 )
#define MIN_Y ( -18 << 12 )

#define MAX_PLATOS 16

//------------------------------------------------------------------------------
//	Plato Enemy
//------------------------------------------------------------------------------
Plato::Plato()
{
	Frame = 0;
	Active = false;	
	Hp = 1; 
	DrawMode = EN_NORMAL;
	
}

Plato::~Plato()
{
}


void Plato::Update()
{
	
	Counter++;
	if( Counter == 60 * 3 )  
	{
		if( Flipped )
		{
			Dy = -Speed;
			Dx = -Speed;
		}
		else
		{
			Dx = -Speed;
			Dy = Speed;
		}
	}
	
	if( Counter == 75 * 3 )  
	{
		Dx = Speed;
		Dy = 0;
	}
	
	x += Dx;
	y += Dy;
	
	if( x < MIN_X )
	{
		Active = false;
		x = 300 << 12;
		y = 96 << 12;
	}

	// animate
	if( ( Counter & 3 ) == 0 )
	{
		Frame = (Frame + 1) & 3;
	}

	Targets::Instance()->AddCoord( x, y );
	
}


	
//------------------------------------------------------------------------------
//	Plato Container
//------------------------------------------------------------------------------
//--------------------------------------
//	Ctor and Dtor
//--------------------------------------
PlatoContainer::PlatoContainer()
{
}
	
PlatoContainer::~PlatoContainer()
{
	Platos.clear();				// clear the bullets
}


void PlatoContainer::Init( int Palette, glImage* const Sprites )
{
	ColorTable = Palette;		// init needed data for drawing
	Images = Sprites;
	Platos.resize( MAX_PLATOS/2 );			// 1/2 of max (speed of iteration)
	Platos.reserve( MAX_PLATOS );		// reserve max for speed.  No need to do this for enemies
}

//--------------------------------------
//	Virtual Functions
//--------------------------------------
void PlatoContainer::UpdateEntities()
{
	
	std::vector<Plato>::iterator iter;
	for( iter = Platos.begin(); iter != Platos.end(); ++iter )
	{
		if( iter->Active )
			iter->Update();
	}
	
}

void PlatoContainer::DrawEntities()
{
	
	glColor( RGB15(31,31,31) );
	numEntities = 0;
	std::vector<Plato>::iterator iter;
	for( iter = Platos.begin(); iter != Platos.end(); ++iter )
	{
		if( iter->Active )
		{
			numEntities++;
			if( (iter->DrawMode == EN_WHITE) )
			{
				iter->DrawMode = EN_NORMAL;
				glColor( RGB15(31,0,0) );
				glSprite( (iter->x >> 12) - 8, (iter->y >> 12) - 8, 
					      GL_FLIP_NONE, &Images[iter->Frame+4] 
					    );
				glColor( RGB15(31,31,31) );
	
			}
			else
			{
				glSprite( (iter->x >> 12) - 8, (iter->y >> 12) - 8, 
						  GL_FLIP_NONE, &Images[iter->Frame+4] 
					    );
			}
			
		}
	}
	
}

bool PlatoContainer::Collide( StarFighter &Star )
{
	std::vector<Plato>::iterator iter;
	for( iter = Platos.begin(); iter != Platos.end(); ++iter )
	{
		
		// star
		if( iter->Active )
		{
			int rad2 = 8 * 8;
			int cx = (iter->x >> 12) - (Star.x >> 12);
			int cy = (iter->y >> 12) - (Star.y >> 12);
			if( ( (cx * cx) + (cy * cy) ) < (rad2 * (1+Star.IsBursting())) )
			{
				if( Star.IsBursting() )
				{
					Vector2df32 p;
					p.x = Star.x;
					p.y = Star.y;
					Particle::Instance()->Spawn( p, 4 );
					Star.IncreaseGauge(15);
					iter->Hp--;
					if( (iter->Hp & 3) == 0 ) Sound::Instance()->PlaySFX(E_SFX_HIT);
					if( iter->Hp < 0 )
					{
						iter->Active = false;
						Star.Score += 33;
						Sound::Instance()->PlaySFX(E_SFX_SMALLEXP);
						// Add some nice explosions
						Explosion::Instance()->Spawn( iter->x >> 12,
													  iter->y >> 12,
													   0,
													   0,
													   Explode::SMALL_4
													 );
							
						iter->x = 600 <<12;
						iter->y = 96 << 12;
						
					}
					if( iter->DrawMode == EN_NORMAL )
					{
						if( (iter->Counter & 1) ) iter->DrawMode = EN_WHITE;
					}
				}
				else
				{
					// kill star
					if( !Star.IsInvincible() ) Star.Kill();
				}
				return true;
			}
		}
		
		
		// missiles if after star enemy is still active
		if( iter->Active )
		{
			int rad2 = 8 * 8;
			
			if( iter->x < ( 256 << 12 ) )
			{
			
				// Missiles
				for( int i = 0; i < MAX_MISSILES; i++)
				{
					HomingMissile Missile= Star.GetMissile(i);
					
					if( Missile.IsActive() )
					{
						int cx = (iter->x >> 12) - (Missile.GetX() >> 12);
						int cy = (iter->y >> 12) - (Missile.GetY() >> 12);
						if( ( (cx * cx) + (cy * cy) ) < (rad2) )
						{
							Vector2df32 p;
							p.x = Missile.GetX();
							p.y = Missile.GetY();
							Particle::Instance()->Spawn( p, 4 );
							Star.DestroyMissile(i);
							iter->Hp--;
							if( iter->Hp < 0 )
							{
								iter->Active = false;
								Star.Score += 33;
								Sound::Instance()->PlaySFX(E_SFX_SMALLEXP);
								// Add some nice explosions
								Explosion::Instance()->Spawn( iter->x >> 12,
															  iter->y >> 12,
															   0,
															   0,
															   Explode::SMALL_4
															 );
									
								iter->x = 600 <<12;
								iter->y = 96 << 12;
								break;
							}
						}
				
					}
				}
			
			}   // iter x < 
	
		}   // iter active
		
	}
	
	return false;
}



void PlatoContainer::KillAllEntities()
{
	std::vector<Plato>::iterator iter;
	for( iter = Platos.begin(); iter != Platos.end(); ++iter )
	{
		if( iter->Active )
		{
			iter->Active = false;
			iter->x = 600 << 12;
			iter->y = 96 << 12;
		}
	}
	
}

//--------------------------------------
//	Non-Virtual Functions
//--------------------------------------
void PlatoContainer::Spawn( int _x, int _y, s32 _speed, int _counter, bool _flip, int _frame )
{
	
	bool SlotFull = true;      		// value to check if all the slots are occupied
	
	std::vector<Plato>::iterator iter;
	for( iter = Platos.begin(); iter != Platos.end(); ++iter )
	{
		if( !iter->Active )		// insert a bullet on a free slot
		{

			iter->x = _x << 12;
			iter->y = _y << 12;
			iter->Dx = _speed;
			iter->Dy = 0;
			iter->Speed = _speed;	
			iter->Flipped = _flip;
			
			iter->Active = true;
			iter->Frame = _frame & 3;
			iter->Counter = _counter;
			iter->Hp = 1;
			SlotFull = false;			// there's a free slot so we dont execute the code below
			break;
		
		}
	}
	
	// All the slots are full
	// Only push back a bullet if the slot is full and we have less than numbullets
	if( SlotFull && (Platos.size() < MAX_PLATOS) )
	{
		Plato popcorn;
		popcorn.x = _x << 12;
		popcorn.y = _y << 12;
		popcorn.Dx = _speed;
		popcorn.Dy = 0;
		popcorn.Speed = _speed;	
		popcorn.Flipped = _flip;
			
		popcorn.Active = true;
		popcorn.Frame = _frame & 3;
		popcorn.Counter = _counter;
		popcorn.Hp = 1;
		Platos.push_back(popcorn);
	}
	
}




