/******************************************************************************
*******************************************************************************

	Bubble Fight EX
	relminator
	http://rel.betterwebber.com


	Waver class


*******************************************************************************
******************************************************************************/

#include "Waver.h"

#define MIN_X ( -20 << 12 )
#define MIN_Y ( -18 << 12 )

#define MAX_WAVERS 16

//------------------------------------------------------------------------------
//	Waver Enemy
//------------------------------------------------------------------------------
Waver::Waver()
{
	Frame = 0;
	Active = false;	
	Hp = 1; 
	DrawMode = EN_NORMAL;
	
}

Waver::~Waver()
{
}


void Waver::Update()
{
	
	Counter++;
	Angle += 256;
	if( (Angle & (BRAD_PI -1)) == 0 )
		Radius = ((sinLerp(Counter * 256) * 60) >>12 );
		
	x += Dx;
	Dy = sinLerp( Angle ) * Radius;
	y = Sy + Dy;
	
	if( x < MIN_X )
	{
		Active = false;
		x = 300 << 12;
		y = 96 << 12;
	}

	// animate
	if( ( Counter & 3 ) == 0 )
	{
		Frame = (Frame + 1) ;
		if( Frame > 5 ) Frame = 0;
	}
	
	if( Active ) Targets::Instance()->AddCoord( x, y );
}


	
//------------------------------------------------------------------------------
//	Waver Container
//------------------------------------------------------------------------------
//--------------------------------------
//	Ctor and Dtor
//--------------------------------------
WaverContainer::WaverContainer()
{
}
	
WaverContainer::~WaverContainer()
{
	Wavers.clear();				// clear the bullets
}


void WaverContainer::Init( int Palette, glImage* const Sprites )
{
	ColorTable = Palette;		// init needed data for drawing
	Images = Sprites;
	Wavers.resize( MAX_WAVERS/2 );			// 1/2 of max (speed of iteration)
	Wavers.reserve( MAX_WAVERS );		// reserve max for speed.  No need to do this for enemies
}

//--------------------------------------
//	Virtual Functions
//--------------------------------------
void WaverContainer::UpdateEntities()
{
	
	std::vector<Waver>::iterator iter;
	for( iter = Wavers.begin(); iter != Wavers.end(); ++iter )
	{
		if( iter->Active )
			iter->Update();
	}
	
}

void WaverContainer::DrawEntities()
{
	
	glColor( RGB15(31,31,31) );
	numEntities = 0;
	std::vector<Waver>::iterator iter;
	for( iter = Wavers.begin(); iter != Wavers.end(); ++iter )
	{
		if( iter->Active )
		{
			numEntities++;
			if( (iter->DrawMode == EN_WHITE) )
			{
				iter->DrawMode = EN_NORMAL;
				glColor( RGB15(31,0,0) );
				glSprite( (iter->x >> 12) - 8, (iter->y >> 12) - 8, 
					      GL_FLIP_NONE, &Images[iter->Frame+24] 
					    );
				glColor( RGB15(31,31,31) );
	
			}
			else
			{
				glSprite( (iter->x >> 12) - 8, (iter->y >> 12) - 8, 
						  GL_FLIP_NONE, &Images[iter->Frame+24] 
					    );
			}
			
		}
	}
	
}

bool WaverContainer::Collide( StarFighter &Star )
{
	std::vector<Waver>::iterator iter;
	for( iter = Wavers.begin(); iter != Wavers.end(); ++iter )
	{
		if( iter->Active )
		{
			int rad2 = 8 * 8;
			int cx = (iter->x >> 12) - (Star.x >> 12);
			int cy = (iter->y >> 12) - (Star.y >> 12);
			if( ( (cx * cx) + (cy * cy) ) < (rad2 * (1+Star.IsBursting())) )
			{
				if( Star.IsBursting() )
				{
					Vector2df32 p;
					p.x = Star.x;
					p.y = Star.y;
					Particle::Instance()->Spawn( p, 2 );
					Star.IncreaseGauge(15);
					iter->Hp--;
					if( (iter->Hp & 3) == 0 ) Sound::Instance()->PlaySFX(E_SFX_HIT);
					if( iter->Hp < 0 )
					{
						iter->Active = false;
						Star.Score += 33;
						Sound::Instance()->PlaySFX(E_SFX_SMALLEXP);
						// Add some nice explosions
						Explosion::Instance()->Spawn( iter->x >> 12,
													  iter->y >> 12,
													   0,
													   0,
													   Explode::SMALL_2
													 );
							
						iter->x = 600 <<12;
						iter->y = 96 << 12;
						
					}
					if( iter->DrawMode == EN_NORMAL )
					{
						if( (iter->Counter & 1) ) iter->DrawMode = EN_WHITE;
					}
				}
				else
				{
					// kill star
					if( !Star.IsInvincible() ) Star.Kill();
				}
				return true;
			}
		}
		
		// missiles if after star enemy is still active
		if( iter->Active )
		{
			int rad2 = 8 * 8;
			
			if( iter->x < ( 256 << 12 ) )
			{
			
				// Missiles
				for( int i = 0; i < MAX_MISSILES; i++)
				{
					HomingMissile Missile= Star.GetMissile(i);
					
					if( Missile.IsActive() )
					{
						int cx = (iter->x >> 12) - (Missile.GetX() >> 12);
						int cy = (iter->y >> 12) - (Missile.GetY() >> 12);
						if( ( (cx * cx) + (cy * cy) ) < (rad2) )
						{
							Vector2df32 p;
							p.x = Missile.GetX();
							p.y = Missile.GetY();
							Particle::Instance()->Spawn( p, 1 );
							Star.DestroyMissile(i);
							iter->Hp--;
							if( iter->Hp < 0 )
							{
								iter->Active = false;
								Star.Score += 33;
								Sound::Instance()->PlaySFX(E_SFX_SMALLEXP);
								// Add some nice explosions
								Explosion::Instance()->Spawn( iter->x >> 12,
															  iter->y >> 12,
															   0,
															   0,
															   Explode::SMALL_4
															 );
									
								iter->x = 600 <<12;
								iter->y = 96 << 12;
								break;
							}
						}
				
					}
				}
			
			}   // iter x < 
	
		}   // iter active
		
	}
	return false;
}



void WaverContainer::KillAllEntities()
{
	std::vector<Waver>::iterator iter;
	for( iter = Wavers.begin(); iter != Wavers.end(); ++iter )
	{
		if( iter->Active )
		{
			iter->Active = false;
			iter->x = 600 << 12;
			iter->y = 96 << 12;
		}
	}
	
}

//--------------------------------------
//	Non-Virtual Functions
//--------------------------------------
void WaverContainer::Spawn( int _x, int _y, s32 _speed, s32 _angle, int _radius, bool _flip, int _frame )
{
	
	bool SlotFull = true;      		// value to check if all the slots are occupied
	
	std::vector<Waver>::iterator iter;
	for( iter = Wavers.begin(); iter != Wavers.end(); ++iter )
	{
		if( !iter->Active )		// insert a bullet on a free slot
		{

			iter->x = _x << 12;
			iter->Sy = _y << 12;
			iter->Dx = _speed;
			iter->Radius = _radius;
			iter->Angle = _angle + _flip * BRAD_PI;
			
			iter->Active = true;
			iter->Frame = _frame & 3;
			iter->Counter = 0;//_flip * BRAD_PI;
			iter->Hp = 1;
			SlotFull = false;			// there's a free slot so we dont execute the code below
			break;
		
		}
	}
	
	// All the slots are full
	// Only push back a bullet if the slot is full and we have less than numbullets
	if( SlotFull && (Wavers.size() < MAX_WAVERS) )
	{
		Waver popcorn;
		popcorn.x = _x << 12;
		popcorn.Sy = _y << 12;
		popcorn.Dx = _speed;
		popcorn.Radius = _radius;
		popcorn.Angle = _angle + _flip * BRAD_PI;
			
		popcorn.Active = true;
		popcorn.Frame = _frame & 3;
		popcorn.Counter = 0;//_flip * BRAD_PI;
		popcorn.Hp = 1;
		Wavers.push_back(popcorn);
	}
	
}




